<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;


class Pla01900ParamCozinhaCentral extends \Zeedhi\Framework\Controller\Report {

    protected $entityManager;
    protected $birtStrategy;

    public function __construct(ReportService $reportService, $entityManager, $birtStrategy) {
        $this->entityManager   = $entityManager;
        $this->birtStrategy    = $birtStrategy;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
    'coz00100' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'CDPRODPRODUCAO' => 'P_PRODUTOACAB',
                'NRORG'      => 'P_NRORG',
                'CDPICTPROD' => 'P_PRODUCTMASK',
                'CDPICTPRAT' => 'P_CDPICTPRAT',
                'LISTASERV'  => 'P_CDSERVFILI',
                'LISTAUNID'  => 'P_CDFILIAL',
                'CDCENTRAL'  => 'P_CDCENTRAL'
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        if (isset($row["CDGRUPSERV"]) and $row["CDGRUPSERV"] !== 'T') {
            $row["CDGRUPSERV"] = $row["CDGRUPSERV"];
        } else if (isset($row["NMGRUPSERV"]) and ($row["NMGRUPSERV"] !== 'T' and count($row["NMGRUPSERV"]) > 0)) {
            $row["CDGRUPSERV"] = join('_',$row["NMGRUPSERV"]);
        } else $row["CDGRUPSERV"] = 'T';

        if (($row['CDFILIAL'] !== 'T' && $row['NMSERVFILI'] === 'T') || ($row['CDFILIAL'] !== 'T' && $row['NMSERVFILI'] !== 'T')) {//servico considera a unidade selecionada
            $resultServ = $this->returnServ($row);  
            $row['LISTASERV'] = implode("_", $resultServ);
            $row['LISTAUNID'] = $row['CDFILIAL'];
        } else if ($row['CDFILIAL'] === 'T' && $row['NMSERVFILI'] !== 'T') {
            $resultServ = $this->returnServ($row);  
            $row['LISTASERV'] = implode("_", $resultServ);
            $row['LISTAUNID'] = 'T';
        } else {
            $row['LISTASERV'] = 'T';
            $row['LISTAUNID'] = 'T';
        }
        
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }  
    
    public function returnServ($row) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
        
        $queryOracle = <<<SQL
            SELECT 
                DISTINCT (S.CDFILIAL || S.CDSERVFILI) AS CDFILISERV,
                S.CDFILIAL,
                F.NMFILIAL,
                S.CDSERVFILI,
                S.NMSERVICO,
                S.NRORG
            FROM 
                SERVICO S 
            JOIN 
                FILIAL F ON S.CDFILIAL = F.CDFILIAL 
            LEFT JOIN 
                FILGRUPFILI FG ON S.CDFILIAL = FG.CDFILIAL AND (FG.CDGRUPFILI = :CDGRUPFILI OR :CDGRUPFILI = 'T') 
            WHERE 
                ((INSTR(:CDFILIAL, S.CDFILIAL) > 0) OR (:CDFILIAL = 'T')) 
                AND ((INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T')) 
                AND (IDTIPOSERV = 'N') 
            ORDER BY 
                CDFILIAL, 
                CDSERVFILI
SQL;

        $queryPostgre = <<<SQL
            SELECT 
                DISTINCT ON (S.CDFILIAL || S.CDSERVFILI) (S.CDFILIAL || S.CDSERVFILI) AS CDFILISERV,
                S.CDFILIAL,
                F.NMFILIAL,
                S.CDSERVFILI,
                S.NMSERVICO,
                S.NRORG
            FROM 
                SERVICO S 
            JOIN 
                FILIAL F ON S.CDFILIAL = F.CDFILIAL 
            LEFT JOIN 
                FILGRUPFILI FG ON S.CDFILIAL = FG.CDFILIAL AND (FG.CDGRUPFILI = :CDGRUPFILI OR :CDGRUPFILI = 'T') 
            WHERE 
                (POSITION(:CDFILIAL IN S.CDFILIAL) > 0 OR :CDFILIAL = 'T') 
                AND (POSITION(:CDGRUPSERV IN S.CDGRUPSERV) <> 0 OR :CDGRUPSERV = 'T') 
                AND (IDTIPOSERV = 'N') 
            ORDER BY 
                (S.CDFILIAL || S.CDSERVFILI),
                S.CDFILIAL, 
                S.CDSERVFILI
SQL;

        $params = array (
	        "CDGRUPFILI" => $row["CDGRUPFILI"],
	        "CDFILIAL"   => $row["CDFILIAL"],
	        "CDGRUPSERV" => $row["CDGRUPSERV"]
        );
        
        $arrayQueries = array(
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        $result = $connection->fetchAll($query, $params);

        $servicos = [];
          foreach($result as $serv){
            $concatServ = $serv['CDSERVFILI'];
            array_push($servicos, $concatServ);
        }
        return $servicos;
    }
}